    {{ name }}.vm.box = "{{ box }}"
    {{ name }}.vm.synced_folder ".", "/vagrant", id: "vagrant-root", disabled: true

    {{ name }}.vm.provider :virtualbox do |domain|
      domain.cpus = 1
      domain.memory = 512
    end
