    {{ name }}.vm.box = "{{ box }}"
    {{ name }}.vm.synced_folder ".", "/vagrant", disabled: true
    {{ name }}.ssh.insert_key = false
    {{ name }}.vm.boot_timeout = 180
    {{ name }}.vm.guest = :freebsd

    {{ name }}.vm.provider :libvirt do |domain|
      domain.nic_adapter_count = 8
      domain.memory = 512
      domain.cpus = 1
      domain.driver = "kvm"
      domain.nic_model_type = "e1000"
    end
